unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TIOTest = (itDIODI, itADDI);

  TForm1 = class(TForm)
    ChecksGroup: TGroupBox;
    DIODICheck: TCheckBox;
    ADDICheck: TCheckBox;
    GoButton: TButton;
    PollTime: TTimer;
    ReportMemo: TMemo;
    procedure StopButtonClick(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
  private
    procedure PrintReport;
    procedure DoReportLine(Header: String; Test: TIOTest);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB;

var
  DIODI, ADDI: LongWord;
  TicksPerSec: Int64;
  SecsPerTick: Double;
  Tests: set of TIOTest;
  Hist: array[TIOTest, 0..20] of Integer;
  TestCt: Integer;
  DIOData: LongWord;
  ADData: array of Double;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  DeviceMask, Status, PID: LongWord;
  OutMask: Byte;

begin
  DIODI := diNone;
  ADDI := diNone;
  DeviceMask := GetDevices;
  for I := 0 to 31 do
    if (DeviceMask and (1 shl I)) <> 0 then begin
      Status := QueryDeviceInfo(I, @PID, nil, nil, nil, nil);
      if Status = ERROR_SUCCESS then begin
        if PID = $8001 then begin
          if DIODI = diNone then DIODI := I;
        end
        else if (PID and $FEC0) = $8040 then begin
          if ADDI = diNone then ADDI := I;
        end;
      end;
    end
  ;

  if (DIODI = diNone) or (DIODI = diNone) then begin
    ReportMemo.Lines.Text := 'This test needs a USB-DIO-32 and a USB-AI* board.';
    if (DIODI = diNone) then ReportMemo.Lines.Add('USB-DIO-32 not found.');
    if ( ADDI = diNone) then ReportMemo.Lines.Add('USB-AI* not found.');
    GoButton.Enabled := False;
    Exit;
  end;

  OutMask := $00;
  DIOData := $FFFFFFFF;
  DIO_Configure(DIODI, False, @OutMask, @DIOData);
  DIO_Configure( ADDI, False, @OutMask, @DIOData);

  SetLength(ADData, 256);
  QueryPerformanceFrequency(TicksPerSec);
  SecsPerTick := 1 / TicksPerSec;
end;

procedure TForm1.GoButtonClick(Sender: TObject);
var
  T: TIOTest;
  I: Integer;

begin
  Tests := [];
  if DIODICheck.Checked then Include(Tests, itDIODI);
  if ADDICheck.Checked then Include(Tests, itADDI);

  TestCt := 0;
  for T := Low(T) to High(T) do
    for I := 0 to High(Hist[T]) do
      Hist[T, I] := 0
  ;

  GoButton.Caption := 'Stop';
  GoButton.OnClick := StopButtonClick;
  DIODICheck.Enabled := False;
  ADDICheck.Enabled := False;
  PollTime.Enabled := True;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  PrintReport;

  PollTime.Enabled := False;
  DIODICheck.Enabled := True;
  ADDICheck.Enabled := True;
  GoButton.Caption := 'Go';
  GoButton.OnClick := GoButtonClick;
end;

procedure ProcessTime(Test: TIOTest; dTicks: Int64);
var
  iT: Integer;
  dT: Double;

begin
  dT := dTicks * SecsPerTick;
  //iT := Round(dT * 10000);
  iT := Round(dT * 10000);
  if iT > High(Hist[Test]) then iT := High(Hist[Test]);
  Inc(Hist[Test, iT]);
end;

procedure TForm1.DoReportLine(Header: String; Test: TIOTest);
var
  I: Integer;
  Buf: String;

begin
  Buf := Header + ':';
  for I := 0 to High(Hist[Test]) do begin
    Buf := Buf + #9'' + IntToStr(Hist[Test, I]);
  end;
  ReportMemo.Lines.Add(Buf);
end;

procedure TForm1.PollTimeTimer(Sender: TObject);
var
  LastTime, ThisTime: Int64;

begin
  if itDIODI in Tests then begin
    QueryPerformanceCounter(LastTime);
    DIO_ReadAll(DIODI, @DIOData);
    QueryPerformanceCounter(ThisTime);
    ProcessTime(itDIODI, ThisTime - LastTime);
  end;

  if itADDI in Tests then begin
    QueryPerformanceCounter(LastTime);
    DIO_ReadAll(ADDI, @DIOData);
    QueryPerformanceCounter(ThisTime);
    ProcessTime(itADDI, ThisTime - LastTime);
  end;

  Inc(TestCt);

  if (TestCt > 1) and ((TestCt mod 10) <> 0) then Exit;

  PrintReport;
end;

procedure TForm1.PrintReport;
var
  Buf: String;
  I: Integer;
  
begin
  ReportMemo.Lines.Text := IntToStr(TestCt) + ' passes';
  Buf := 'Microseconds';
  for I := 0 to High(Hist[itDIODI]) do
    Buf := Buf + #9 + FloatToStr(I * 100)
  ;
  Buf := Buf + '+'#13#10;
  ReportMemo.Lines.Add(Buf);
  if itDIODI in Tests then DoReportLine('DIO board DI', itDIODI);
  if itADDI  in Tests then DoReportLine('ADC board DI', itADDI );
end;

end.
